/*
       *  Licensed Materials - Property of IBM
       *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
       *  US Government Users Restricted Rights - Use, duplication or
       *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

function wlCommonInit(){
	// Common initialization code goes here

	if (WL.Client.getEnvironment() === 'iphone' ||
	WL.Client.getEnvironment() === 'ipad' ||
	WL.Client.getEnvironment() === 'android') {

	//gets created when we initialize the collection.
		var usersCollection = null,

		//Cache variables in the DOM
		statusTag = WLJQ('div#status'),
		usernameTag = WLJQ('input#username'),
		passwordTag = WLJQ('input#password'),
		passwordOldTag = WLJQ('input#password_old'),
		passwordNewTag = WLJQ('input#password_new'),
		nameTag = WLJQ('input#name'),
		ageTag = WLJQ('input#age'),
		idTag = WLJQ('input#id'),
		enhanceTag = WLJQ('input#inp_enhance'),
		tableTag = WLJQ('table#user_table'),

		//Logs messages to the console and to a div tag
		logMessage = function (msg) {
			WL.Logger.debug(msg);
			statusTag.text(msg);
			tableTag.html('');
		},

		//Simple function that check if obj is null or undefined
		checkUndefOrNull = function (obj) {
			return (typeof obj === 'undefined') || obj === null;
		},
		//Simple function that check if collection is init
		checkColInit = function (obj) {
			if ((typeof obj === 'undefined') || obj === null){
				logMessage('You must initialize the collection first.');
				return false;
			}else{ return true; }
		},

		//Generic callback called when there's a failure
		genericFailureCallback = function  (err) {
			logMessage(err + ' Error Message: ' + WL.JSONStore.getErrorMessage(err));
		};
		//END DEFINE VARS


		//Initializes the 'users' collection
		WLJQ('button#initCollection').bind('click', function () {

				WL.Logger.debug('Called button#initCollection');

				var usr = usernameTag.val(),
				pwd = passwordTag.val(),
				collectionName = 'users',
				usersSearchFields = {"age":"integer","name":"string"},
				usersAdapterOptions = {
					name: 'user',
					replace: 'updateUser',
					remove: 'deleteUser',
					add: 'addUser',
					load: {
						procedure: 'getUsers',
						params: [],
						key: 'users'
					},
					accept: function (data) {
						return (data.status === 200);
					}
				};


				var initCollectionSuccessCallback = function () {
					WL.Logger.debug('WL.usersCollection is ready');
					logMessage('Collection has been initialized');
					//$.event.trigger('usersCollection/ready');

					//Split people items and create orders collection

				};

				var options = {adapter: usersAdapterOptions,
						onSuccess: initCollectionSuccessCallback,
						onFailure: genericFailureCallback,
						load:true};
				
				
				if (pwd.length > 0) {

					options.password = pwd;
					
					WL.Logger.debug('Using Password');

					passwordTag.val('');

				} else {

					WL.Logger.debug('Using NO Password');
				}
				
				if(usr.length > 0) {
					WL.Logger.debug('Using Username: ' + usr);
					options.username = usr;
				}
				
				usersCollection = WL.JSONStore.initCollection(
					"users",
					usersSearchFields, options),

				getCollectionInstance = function () {

					return users;

				};

				//Public API
				return {
					getInstance : getCollectionInstance
				};
		});//END initCollection

		//Convenience function to load the table with entries
		var reloadTable = function(results) {

				var msg = '<tr><td>ID</td><td>NAME</td><td>AGE</td></tr>',
					i = results.length,
					curr;

				if ( i < 1) {

					logMessage('No Entries Found');

				} else {

					while ( i-- ) {
						curr = results[i].json;
						id = results[i]._id;
						msg += '<tr><td>' + id + '</td><td>' + curr.name + '</td><td>' + curr.age + '</td></tr>';
					}

					WL.Logger.debug('Find Raw Results: ' + JSON.stringify(results));
					logMessage(''); //Clear Log
					tableTag.html(msg); //Add Table
				}

		};

		//Does a findAll on the 'users' collection
		WLJQ('button#find').bind('click', function () {

			WL.Logger.debug('Called button#find');
			if (!checkColInit(usersCollection)) {return;}

			usersCollection.findAll({onFailure: genericFailureCallback, onSuccess: function (results) {
				reloadTable(results);
			}});//END findAll

		});//END findAll click


		//Changes the users password
		WLJQ('button#change_password').bind('click', function () {

			WL.Logger.debug('Called button#change_password');

			var password_old = passwordOldTag.val(),
				password_new = passwordNewTag.val();

			if (password_old.length < 1 || password_new.length < 1 ||
				checkUndefOrNull(usersCollection)) {
				logMessage('You must enter your old password as well as the new one.');
			} else {

				var win =   function (data) {
					logMessage('Password was successfully changed : ' + data);
					passwordOldTag.val('');
					passwordNewTag.val('');
				};

				var options = {onSuccess: win, onFailure: genericFailureCallback};

				WL.JSONStore.changePassword(password_old,password_new,options);
			}
		});//END Change Password


		//Closes the JSONStore
		WLJQ('button#close_all').bind('click', function () {

			WL.Logger.debug('Called button#close_all');

			WL.JSONStore.closeAll({onSuccess: function() {
				logMessage('JSONStore closed');
			}, onFailure: function () {
				logMessage('Failure trying to close JSONStore');
			}});
			
		});//END Close All


		//Replaces a user by his or her name
		WLJQ('button#replace').bind('click', function () {

			WL.Logger.debug('Called button#replace');
			if (!checkColInit(usersCollection)) {return;}

			var name = nameTag.val(),
				id = idTag.val(),
				age = ageTag.val();

			if (id.length < 1 ) {
				logMessage('You must provide a valid value for ID');
			} else {

				var doc = WL.JSONStore.documentify(id, {name: name, age: age});

				var win =   function (data) {
					logMessage('Replaced ' + data + ' document.');
					//Clear Text Boxes
					nameTag.val('');
					ageTag.val('');
					idTag.val('');
				};

				var options = {onSuccess: win, onFailure: genericFailureCallback};

				usersCollection.replace(doc,options);
			}
		});

		//Enhance
		WLJQ('button#enhance').bind('click', function () {

			WL.Logger.debug('Called button#enhance');

			var name = enhanceTag.val();

			if ( name.length < 1 ) {
				logMessage('You must provide a valid search value');
				return;
			}

			//Create a custom function
			var custFunction = function (user_name,onSuccess) {

				var query = {name: user_name};

				var win =  function (data) {
					onSuccess(data);
				};

				var fail =  function (data) {
					genericFailureCallback(data);
				};

				var options = {onSuccess: win, onFailure: fail};
				usersCollection.find(query, options);
			};

			usersCollection.enhance('getUserByName', custFunction);

			var onSuccess = function (data) {
				reloadTable(data);
				enhanceTag.val('');
			};

			usersCollection.getUserByName(name, onSuccess);
		});//END Enhance


		//Removes the internally stored collection
		WLJQ('button#remove_collection').bind('click', function () {

			WL.Logger.debug('Called button#remove_collection');

			var win =  function (data) {
				if (data === 0) {
					logMessage('Users collection was removed');
				}else{
					logMessage('Users Collection was not removed');
				}
			};

			var options = {onSuccess: win, onFailure: genericFailureCallback};

			usersCollection.removeCollection(options);
		});


		//Destroys the internal storage linked to the JSONStore
		WLJQ('button#destroy').bind('click', function () {

			WL.Logger.debug('Called button#destroy');

			WL.JSONStore.destroy({onFailure: genericFailureCallback, onSuccess: function () {

				logMessage('JSONStore was destroyed');
			}});
		});

	}//end if statement that checks for valid environments
}
